@@ -13,7 +13,6 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import base64
 import gzip
 import mock
 import os
@@ -23,6 +22,7 @@ import tempfile
 
 from oslo_concurrency import processutils
 from oslo_config import cfg
+from oslo_serialization import base64
 from oslo_service import loopingcall
 from oslo_utils import imageutils
 from oslotest import base as test_base
@@ -227,6 +227,71 @@ class WorkOnDiskTestCase(test_base.BaseTestCase):
                                              boot_mode="bios",
                                              disk_label='gpt')
 
+    @mock.patch.object(disk_utils, 'block_uuid', autospec=True)
+    @mock.patch.object(disk_utils, 'populate_image', autospec=True)
+    @mock.patch.object(utils, 'mkfs', autospec=True)
+    def test_uefi_localboot(self, mock_mkfs, mock_populate_image,
+                            mock_block_uuid):
+        """Test that we create a fat filesystem with UEFI localboot."""
+        root_part = '/dev/fake-part1'
+        efi_part = '/dev/fake-part2'
+        self.mock_mp.return_value = {'root': root_part,
+                                     'efi system partition': efi_part}
+        self.mock_ibd.return_value = True
+        mock_ibd_calls = [mock.call(root_part),
+                          mock.call(efi_part)]
+
+        disk_utils.work_on_disk(self.dev, self.root_mb,
+                                self.swap_mb, self.ephemeral_mb,
+                                self.ephemeral_format,
+                                self.image_path, self.node_uuid,
+                                boot_option="local", boot_mode="uefi")
+
+        self.mock_mp.assert_called_once_with(self.dev, self.root_mb,
+                                             self.swap_mb, self.ephemeral_mb,
+                                             self.configdrive_mb,
+                                             self.node_uuid, commit=True,
+                                             boot_option="local",
+                                             boot_mode="uefi",
+                                             disk_label=None)
+        self.assertEqual(self.mock_ibd.call_args_list, mock_ibd_calls)
+        mock_mkfs.assert_called_once_with(fs='vfat', path=efi_part,
+                                          label='efi-part')
+        mock_populate_image.assert_called_once_with(self.image_path,
+                                                    root_part)
+        mock_block_uuid.assert_any_call(root_part)
+        mock_block_uuid.assert_any_call(efi_part)
+
+    @mock.patch.object(disk_utils, 'block_uuid', autospec=True)
+    @mock.patch.object(disk_utils, 'populate_image', autospec=True)
+    @mock.patch.object(utils, 'mkfs', autospec=True)
+    def test_preserve_ephemeral(self, mock_mkfs, mock_populate_image,
+                                mock_block_uuid):
+        """Test that ephemeral partition doesn't get overwritten."""
+        ephemeral_part = '/dev/fake-part1'
+        root_part = '/dev/fake-part2'
+        ephemeral_mb = 256
+        ephemeral_format = 'exttest'
+
+        self.mock_mp.return_value = {'ephemeral': ephemeral_part,
+                                     'root': root_part}
+        self.mock_ibd.return_value = True
+        calls = [mock.call(root_part),
+                 mock.call(ephemeral_part)]
+        disk_utils.work_on_disk(self.dev, self.root_mb,
+                                self.swap_mb, ephemeral_mb, ephemeral_format,
+                                self.image_path, self.node_uuid,
+                                preserve_ephemeral=True)
+        self.assertEqual(self.mock_ibd.call_args_list, calls)
+        self.mock_mp.assert_called_once_with(self.dev, self.root_mb,
+                                             self.swap_mb, ephemeral_mb,
+                                             self.configdrive_mb,
+                                             self.node_uuid, commit=False,
+                                             boot_option="netboot",
+                                             boot_mode="bios",
+                                             disk_label=None)
+        self.assertFalse(mock_mkfs.called)
+
 
 @mock.patch.object(utils, 'execute', autospec=True)
 class MakePartitionsTestCase(test_base.BaseTestCase):
@@ -574,7 +639,7 @@ class GetConfigdriveTestCase(test_base.BaseTestCase):
                           'http://1.2.3.4/cd', 'fake-node-uuid')
         self.assertFalse(mock_copy.called)
 
-    @mock.patch.object(base64, 'b64decode', autospec=True)
+    @mock.patch.object(base64, 'decode_as_bytes', autospec=True)
     def test_get_configdrive_base64_error(self, mock_b64, mock_requests,
                                           mock_copy):
         mock_b64.side_effect = TypeError
@@ -635,7 +700,8 @@ class OtherFunctionTestCase(test_base.BaseTestCase):
         disk_utils.qemu_img_info('img')
         path_exists_mock.assert_called_once_with('img')
         execute_mock.assert_called_once_with('env', 'LC_ALL=C', 'LANG=C',
-                                             'qemu-img', 'info', 'img')
+                                             'qemu-img', 'info', 'img',
+                                             prlimit=mock.ANY)
         qemu_img_info_mock.assert_called_once_with('out')
 
     @mock.patch.object(utils, 'execute', autospec=True)
@@ -643,7 +709,8 @@ class OtherFunctionTestCase(test_base.BaseTestCase):
         disk_utils.convert_image('source', 'dest', 'out_format')
         execute_mock.assert_called_once_with('qemu-img', 'convert', '-O',
                                              'out_format', 'source', 'dest',
-                                             run_as_root=False)
+                                             run_as_root=False,
+                                             prlimit=mock.ANY)
 
     @mock.patch.object(os.path, 'getsize', autospec=True)
     @mock.patch.object(disk_utils, 'qemu_img_info', autospec=True)
@@ -704,7 +771,7 @@ class OtherFunctionTestCase(test_base.BaseTestCase):
                                              use_standard_locale=True)
 
 
-@mock.patch.object(utils, 'execute')
+@mock.patch.object(utils, 'execute', autospec=True)
 class WholeDiskPartitionTestCases(test_base.BaseTestCase):
 
     def setUp(self):
@@ -714,16 +781,17 @@ class WholeDiskPartitionTestCases(test_base.BaseTestCase):
         self.node_uuid = "12345678-1234-1234-1234-1234567890abcxyz"
 
     def test_get_partition_present(self, mock_execute):
-        blkid_output = '/dev/fake12\n'
-        mock_execute.side_effect = [(None, ''), (blkid_output, '')]
+        lsblk_output = 'NAME="fake12" LABEL="config-2"\n'
+        part_result = '/dev/fake12'
+        mock_execute.side_effect = [(None, ''), (lsblk_output, '')]
         result = disk_utils._get_labelled_partition(self.dev,
                                                     self.config_part_label,
                                                     self.node_uuid)
-        self.assertEqual(blkid_output.rstrip(), result)
+        self.assertEqual(part_result, result)
         execute_calls = [
             mock.call('partprobe', self.dev, run_as_root=True),
-            mock.call('blkid', '-o', 'device', self.dev, '-t',
-                      'LABEL=config-2', check_exit_code=[0, 2],
+            mock.call('lsblk', '-Po', 'name,label', self.dev,
+                      check_exit_code=[0, 1],
                       use_standard_locale=True, run_as_root=True)
         ]
         mock_execute.assert_has_calls(execute_calls)
@@ -737,27 +805,31 @@ class WholeDiskPartitionTestCases(test_base.BaseTestCase):
         self.assertIsNone(result)
         execute_calls = [
             mock.call('partprobe', self.dev, run_as_root=True),
-            mock.call('blkid', '-o', 'device', self.dev, '-t',
-                      'LABEL=config-2', check_exit_code=[0, 2],
+            mock.call('lsblk', '-Po', 'name,label', self.dev,
+                      check_exit_code=[0, 1],
                       use_standard_locale=True, run_as_root=True)
         ]
         mock_execute.assert_has_calls(execute_calls)
 
     def test_get_partition_DeployFail_exc(self, mock_execute):
-        blkid_output = '/dev/fake12\n/dev/fake13\n'
-        mock_execute.side_effect = [(None, ''), (blkid_output, '')]
-        self.assertRaises(exception.InstanceDeployFailure,
-                          disk_utils._get_labelled_partition, self.dev,
-                          self.config_part_label, self.node_uuid)
+        label = 'config-2'
+        lsblk_output = ('NAME="fake12" LABEL="%s"\n'
+                        'NAME="fake13" LABEL="%s"\n' %
+                        (label, label))
+        mock_execute.side_effect = [(None, ''), (lsblk_output, '')]
+        self.assertRaisesRegex(exception.InstanceDeployFailure,
+                               'fake .*fake12 .*fake13',
+                               disk_utils._get_labelled_partition, self.dev,
+                               self.config_part_label, self.node_uuid)
         execute_calls = [
             mock.call('partprobe', self.dev, run_as_root=True),
-            mock.call('blkid', '-o', 'device', self.dev, '-t',
-                      'LABEL=config-2', check_exit_code=[0, 2],
+            mock.call('lsblk', '-Po', 'name,label', self.dev,
+                      check_exit_code=[0, 1],
                       use_standard_locale=True, run_as_root=True)
         ]
         mock_execute.assert_has_calls(execute_calls)
 
-    @mock.patch.object(disk_utils.LOG, 'error')
+    @mock.patch.object(disk_utils.LOG, 'error', autospec=True)
     def test_get_partition_exc(self, mock_log, mock_execute):
         mock_execute.side_effect = processutils.ProcessExecutionError
         self.assertRaisesRegex(exception.InstanceDeployFailure,
@@ -789,7 +861,7 @@ class WholeDiskPartitionTestCases(test_base.BaseTestCase):
                                                       blk_out=blkid_out)
         self.assertTrue(ret)
 
-    @mock.patch.object(disk_utils.LOG, 'error')
+    @mock.patch.object(disk_utils.LOG, 'error', autospec=True)
     def test_is_disk_larger_than_max_size_exc(self, mock_log, mock_execute):
         mock_execute.side_effect = processutils.ProcessExecutionError
         self.assertRaisesRegex(exception.InstanceDeployFailure,
@@ -823,7 +895,7 @@ class WholeDiskPartitionTestCases(test_base.BaseTestCase):
                                              use_standard_locale=True,
                                              run_as_root=True)
 
-    @mock.patch.object(disk_utils.LOG, 'error')
+    @mock.patch.object(disk_utils.LOG, 'error', autospec=True)
     def test_is_disk_gpt_partitioned_exc(self, mock_log, mock_execute):
         mock_execute.side_effect = processutils.ProcessExecutionError
         self.assertRaisesRegex(exception.InstanceDeployFailure,
@@ -862,7 +934,7 @@ or continue with the current setting?
                                              use_standard_locale=True,
                                              run_as_root=True)
 
-    @mock.patch.object(disk_utils.LOG, 'error')
+    @mock.patch.object(disk_utils.LOG, 'error', autospec=True)
     def test_fix_gpt_structs_exc(self, mock_log, mock_execute):
         mock_execute.side_effect = processutils.ProcessExecutionError
         self.assertRaisesRegex(exception.InstanceDeployFailure,
@@ -1055,13 +1127,13 @@ class WholeDiskConfigDriveTestCases(test_base.BaseTestCase):
             self.assertEqual(1, mock_log.call_count)
             parted_call = mock.call('parted', '-a', 'optimal', '-s',
                                     '--', self.dev, 'mkpart',
-                                    'primary', 'ext2', 2097087,
+                                    'primary', 'fat32', 2097087,
                                     2097151, run_as_root=True)
         else:
             self.assertEqual(0, mock_log.call_count)
             parted_call = mock.call('parted', '-a', 'optimal', '-s',
                                     '--', self.dev, 'mkpart',
-                                    'primary', 'ext2', '-64MiB',
+                                    'primary', 'fat32', '-64MiB',
                                     '-0', run_as_root=True)
         mock_execute.assert_has_calls([
             parted_call,
@@ -1145,7 +1217,7 @@ class WholeDiskConfigDriveTestCases(test_base.BaseTestCase):
         mock_get_configdrive.assert_called_with(config_url, self.node_uuid)
         mock_execute.assert_called_with('parted', '-a', 'optimal', '-s', '--',
                                         self.dev, 'mkpart', 'primary',
-                                        'ext2', '-64MiB', '-0',
+                                        'fat32', '-64MiB', '-0',
                                         run_as_root=True)
         self.assertEqual(2, mock_list_partitions.call_count)
         mock_is_disk_gpt.assert_called_with(self.dev, self.node_uuid)
@@ -1210,7 +1282,7 @@ class WholeDiskConfigDriveTestCases(test_base.BaseTestCase):
         mock_get_configdrive.assert_called_with(config_url, self.node_uuid)
         mock_execute.assert_called_with('parted', '-a', 'optimal', '-s', '--',
                                         self.dev, 'mkpart', 'primary',
-                                        'ext2', '-64MiB', '-0',
+                                        'fat32', '-64MiB', '-0',
                                         run_as_root=True)
         self.assertEqual(1, mock_list_partitions.call_count)
         mock_is_disk_gpt.assert_called_with(self.dev, self.node_uuid)
